@@ -1,5 +1,5 @@
+import hdroller.countdown
 import hdroller.math
-import hdroller.die_repeater
 
 from collections import defaultdict
 from functools import cached_property, lru_cache
@@ -80,16 +80,20 @@ class Die(metaclass=DieType):
                 raise ValueError('Weights must have exactly one dimension.')
             self._min_outcome = min_outcome
         
-        total_weight = numpy.sum(self._weights)
+        total_weight = self._total_weight_no_cache()
         if total_weight >= hdroller.math.MAX_INT_FLOAT:
             self._weights /= total_weight
         
         self._weights.setflags(write=False)
     
     # Cached values.
+    def _total_weight_no_cache(self):
+        # This is used instead of numpy.sum to ensure consistency with cweights/ccweights.
+        return numpy.cumsum(self._weights)[-1]
+    
     @cached_property
     def _total_weight(self):
-        return numpy.sum(self._weights)
+        return self._total_weight_no_cache()
     
     @cached_property
     def _pmf(self):
@@ -129,10 +133,6 @@ class Die(metaclass=DieType):
         result.setflags(write=False)
         return result
     
-    @cached_property
-    def _repeater(self):
-        return hdroller.die_repeater.DieRepeater(self)
-    
     # Creation.
     
     @staticmethod
@@ -532,27 +532,42 @@ class Die(metaclass=DieType):
         result = half_result + half_result
         if num_dice % 2: result += self
         return result
+    
+    def keep(self, num_dice, keep_indexes, max_outcomes=None):
+        """
+        Returns a Die representing:
+        Roll this Die `num_dice` times, sort them (in ascending order) and sum the dice in `keep_indexes`.
+        max_outcomes: If provided, this limits the maximum outcomes of individual dice.
+        """
+        return hdroller.countdown.keep(num_dice, keep_indexes, die=self, max_outcomes=max_outcomes)
         
-    def keep_highest(self, num_dice, num_keep=1):
+    def keep_highest(self, num_dice, num_keep=1, num_drop=0, max_outcomes=None):
         """
         Returns a Die representing:
         Roll this Die `num_dice` times and sum the `num_keep` highest.
+        num_drop: If provided, this many highest dice will be dropped before keeping.
+        max_outcomes: If provided, this limits the maximum outcomes of individual dice.
         """
-        return self._repeater.keep_highest(num_dice, num_keep)
+        if num_keep == 0:
+            return Die(0)
+        start = -(num_keep + (num_drop or 0))
+        stop = -num_drop if num_drop > 0 else None
+        keep = slice(start, stop)
+        return hdroller.countdown.keep(num_dice, keep, die=self, max_outcomes=max_outcomes)
         
-    def keep_lowest(self, num_dice, num_keep=1):
+    def keep_lowest(self, num_dice, num_keep=1, num_drop=0, max_outcomes=None):
         """
         Returns a Die representing:
         Roll this Die `num_dice` times and sum the `num_keep` lowest.
+        num_drop: If provided, this many lowest dice will be dropped before keeping.
+        max_outcomes: If provided, this limits the maximum outcomes of individual dice.
         """
-        return self._repeater.keep_lowest(num_dice, num_keep)
-        
-    def keep_index(self, num_dice, index):
-        """
-        Returns a Die representing:
-        Roll this Die `num_dice` times and take the `index`th from the bottom.
-        """
-        return self._repeater.keep_index(num_dice, index)
+        if num_keep == 0:
+            return Die(0)
+        start = num_drop if num_drop > 0 else None
+        stop = num_keep + (num_drop or 0)
+        keep = slice(start, stop)
+        return hdroller.countdown.keep(num_dice, keep, die=self, max_outcomes=max_outcomes)
 
     # Comparators. These return a Die.
     
@@ -715,7 +730,7 @@ class Die(metaclass=DieType):
         
         return Die.mix(non_explode_die, explode_die, mix_weights=mix_weights)
     
-    def reroll(self, *, outcomes=None, max_times=None):
+    def reroll(self, outcomes=None, max_times=None):
         """
         Rerolls the given outcomes.
         outcomes: Selects which outcomes to reroll. Options:
